/*===============================<o>=====================================

  This software was written by Ian L. Kaplan, Chief Fat Bear, Bear Products
  International.  Use of this software, for any purpose, is granted on two
  conditions: 

   1. This copyright notice must be included with the software
      or any software derived from it.

   2. The risk of using this software is accepted by the user.  No
      warranty as to its usefulness or functionality is provided
      or implied.  The author and Bear Products International provides
      no support.

   July 1996

 ===============================<o>=====================================*/

/*

  This file contains class functions for the pyr_display object.  A
  power 2 Sierpinski pyramid is created by default, by the class
  constructor.  This pyramid is displayed when the Win32 window comes
  up.  The user can select a different power pyramid via the "Sierpinski
  Power" menu item.

  The class functions in pyr_display take care of all the openGL related
  operations.

 */

#include <assert.h>
#include "stdafx.h"

#include "pyrdisp.h"


//
// pyr_display constructor
//
pyr_display::pyr_display(void)
{
  pyr = new Sierpinski( 2 );
  init_spin_vars();
} // pyr_display



//
// init_spin_vars
//
// Initialize the variables that keep track of the current
// rotation angle (SpinAngle) and the axis of rotation.
//
void pyr_display::init_spin_vars(void)
{
  AxisPoint[ 0 ] = On;    // x-axis
  AxisPoint[ 1 ] = Off;   // y-axis
  AxisPoint[ 2 ] = Off;   // z-axis.
  AxisIx = 0;
  SpinAngle = 0;
} // init_spin_vars




//
// incr_spin_vars
//
// Increment the spin angle.  If the pyramid has already been
// rotated through 360 degrees on one axis, rotate it through
// the next axis.  This function is called as part of the OnIdle
// processing.
//
void pyr_display::incr_spin_vars(void)
{
  SpinAngle = SpinAngle + 2;
  if (SpinAngle > 360) {
    AxisPoint[ AxisIx ] = Off;
    AxisIx = (AxisIx + 1) % 3;
    AxisPoint[ AxisIx ] = On;
    SpinAngle = SpinAngle - 360;
  }
} // pyr_display::incr_spin_vars



//
// pyr_display::set_GL_size
//
// Scale the GL display relative to the window.
//
void pyr_display::set_GL_size(void)
{
  int pyr_size = pyr->get_size();
  // extra is the amount of room on the sides of the window
  int extra = pyr_size / 10;
  if (extra < 5)
    extra = 5;
  int AxisMax = pyr_size + extra;

  glViewport(0, 0, cur_width, cur_height);

  glMatrixMode (GL_PROJECTION);
  glLoadIdentity();

  if (cur_width <= cur_height) {
    float AdjY = AxisMax * (GLfloat)cur_height / (GLfloat)cur_width;
    glOrtho(-AxisMax, AxisMax,    // x-axis
	    -AdjY, AdjY,          // y-axis
	    -AxisMax, AxisMax);   // z-axis
  }
  else {
    float AdjX = AxisMax * (GLfloat)cur_height / (GLfloat)cur_width;
    glOrtho(-AdjX, AdjX,          // x-axis
	    -AxisMax, AxisMax,    // y-axis
	    -AxisMax, AxisMax);   // z-axis
  }

  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();  
} // set_GL_size



//
// build_display_list
//
// Convert the polygon face list for the Sierpinski pyramid into 
// an openGL display list.  Display lists are somewhat faster under
// stock Windows NT and considerably faster on systems with hardware
// support for openGL
//
void pyr_display::build_display_list(void)
{
  typedef int point[ 3 ];
  const int size_of_point = sizeof( point )/sizeof(int);
  const int points_per_face = 4;
  int *tmp;
  int i, j;


  tmp = pyr->get_facelist();

  sierpinski_list = glGenLists( 1 );

  assert( sierpinski_list != 0);

  glNewList( sierpinski_list, GL_COMPILE );

  for (j = 0; j < pyr->get_num_faces(); j++) {
    glBegin( GL_POLYGON );
    for (i = 0; i < points_per_face; i++) {
      if (i < points_per_face-1 || point_neq( tmp, tmp - size_of_point)) {
	glVertex3iv(tmp);
      }
      tmp += size_of_point;
    } // i
    glEnd();
  } // j

  glEndList();

} /* build_display_list */



//
// Display the wire frame described by the face list pointed to by
// CurObj.
//
void pyr_display::draw_object(void)
{
  if (sierpinski_list != 0) {

    /* Clear the OpenGL drawing area to black */
    glClearColor((float)0.0, (float)0.0, (float)0.0, (float)0.0);

    /* set line drawing color to white */
    glColor3f((float)1.0, (float)1.0, (float)1.0);
    glPolygonMode( GL_FRONT_AND_BACK, GL_LINE );

    glClear(GL_COLOR_BUFFER_BIT);
    
    glPushMatrix();
    
    glRotatef((float)SpinAngle, (float)AxisPoint[0],
	      (float)AxisPoint[1],
	      (float)AxisPoint[2]);
    
    glCallList( sierpinski_list );
    
    glPopMatrix();
    glFlush();
  }
}  // pyr_display::draw_object




//
// pyr_display::new_pyramid
//
// + delete the old Sierpinski object and create a new one of
//   power "power".
// + Delete the old display list.
// + Create a new display list for the new Sierpinski pyramid.
// + Initialize the global rotation variables.
// + Invalidate the window so that it will be redrawn.
//
void pyr_display::new_pyramid( unsigned int power )
{
  // if there is an existing pyramid, deallocate it
  if (pyr != NULL) {
    delete pyr;
  }

  // allocate a new pyramid
  pyr = new Sierpinski( power );

  // delete the current display list
  glDeleteLists( sierpinski_list, 1 );
  sierpinski_list = 0;
  
  // build a new display list
  build_display_list();

  // initialize rotation variables
  init_spin_vars();

  // scale the GL display for the window
  set_GL_size();
}  // new_pyramid

